#! /usr/bin/python
"""

@ R.B.Young
Created on 30 Aug 2021
"""

from ete3 import NCBITaxa
import sys
import os
import csv
import argparse
parser = argparse.ArgumentParser(description='average OD data and normalise output')
parser.add_argument('-t', metavar='FILENAME', required=True, type=str, help="the truth file")
parser.add_argument('-k', metavar='FILENAME', required=True, type=str, help="the kraken REPORT query file (with the list of taxids)")
args = parser.parse_args()

ncbi = NCBITaxa()

krakenfile = open(args.k, 'r')
taxidlist = []


ranklist = ['phylum', 'class', 'order', 'family', 'genus', 'species']
phyla = []
class_ = []
order = []
family = []
genera = []
species = []


#counts incorrect and correct taxid assignment by the tool
def correctvsincorrect(l1, l2):
	xcorrect = 0
	xincorrect = 0
	for y in l1:
		if int(y) in l2:
			xcorrect += 1
		else:
			xincorrect += 1
	return([xcorrect, xincorrect])


#first make sure that the correct truth file was given 
if ((args.t).split("/")[-1]).split("_TAXID")[0] == ((args.k).split("/")[-1]).split("_REPORT")[0]:
	print("REPORT and truth file match...starting analysis")
else:
	sys.exit('\033[91m' + "Truth file does not match report file" + '\033[0m')
	

#open the kraken report file and read the taxid column
reader = csv.reader(krakenfile, delimiter = "\t")
for line in reader:
	rowcount = 0
	for x in line:
		if rowcount == 4:
			if x not in taxidlist:
				taxidlist.append(x)
		rowcount = rowcount + 1


for taxid in taxidlist:
	if taxid not in ["0"]:
		taxid = int(taxid)
		lineage = ncbi.get_lineage(taxid)
		iddict = ncbi.get_rank(lineage)
		lineagedict = {k:v for k,v in iddict.items() if v in ranklist}

		for ID, tax in lineagedict.items():
			if tax == "phylum":
				if ID not in phyla:
					phyla.append(ID)
			if tax == "class":
				if ID not in class_:
					class_.append(ID)

			if tax == "order":
				if ID not in order:
					order.append(ID)

			if tax == "family":
				if ID not in family:
					family.append(ID)

			if tax == "genus":
				if ID not in genera:
					genera.append(ID)

			if tax == "species":
				if ID not in species:
					species.append(ID)

truthphyla = []
truthclass =[]
truthorder =[]
truthfamily = []
truthgenera = []
truthspecies = []

truthfile = open(args.t, 'r')
reader2 = csv.reader(truthfile)
linecount = 0
for line in reader2:
	if linecount > 0:
		truthtaxa = int(line[9])
		lineage_truth = ncbi.get_lineage(truthtaxa)
		iddict_truth = ncbi.get_rank(lineage_truth)
		truthdict = {k:v for k,v in iddict_truth.items() if v in ranklist}

		for ID, tax in truthdict.items():
			if tax == "phylum":
				if ID not in truthphyla:
					truthphyla.append(ID)
			if tax == "class":
				if ID not in truthclass:
					truthclass.append(ID)

			if tax == "order":
				if ID not in truthorder:
					truthorder.append(ID)

			if tax == "family":
				if ID not in truthfamily:
					truthfamily.append(ID)

			if tax == "genus":
				if ID not in truthgenera:
					truthgenera.append(ID)

			if tax == "species":
				if ID not in truthspecies:
					truthspecies.append(ID)
	linecount = linecount + 1

	if line[11] == "FALSE":
		if line[10]!= "":
			lineagetest = ncbi.get_lineage(int(line[9]))
			if int(line[10]) not in lineagetest:
				print("The following line taxids have an issue: " + str(line))


	

phylavalues = correctvsincorrect(phyla, truthphyla)
speciesvalues = correctvsincorrect(species, truthspecies)
classvalues = correctvsincorrect(class_, truthclass)
ordervalues = correctvsincorrect(order, truthorder)
genusvalues = correctvsincorrect(genera, truthgenera)
familyvalues = correctvsincorrect(family, truthfamily)


#RECALL
phylarecall = int(phylavalues[0])/int(len(truthphyla))
classrecall = int(classvalues[0])/int(len(truthclass))
orderrecall = int(ordervalues[0])/int(len(truthorder))
familyrecall = int(familyvalues[0])/int(len(truthfamily))
genusrecall = int(genusvalues[0])/int(len(truthgenera))
speciesrecall =int(speciesvalues[0])/int(len(truthspecies))

#PRECISION

phylaprecision = int(phylavalues[0])/(int(phylavalues[0]) + int(phylavalues[1]))
classprecision = int(classvalues[0])/(int(classvalues[0]) + int(classvalues[1]))
orderprecision = int(ordervalues[0])/(int(ordervalues[0]) + int(ordervalues[1]))
familyprecision = int(familyvalues[0])/(int(familyvalues[0]) + int(familyvalues[1]))
genusprecision = int(genusvalues[0])/(int(genusvalues[0]) + int(genusvalues[1]))
speciesprecision = int(speciesvalues[0])/(int(speciesvalues[0]) + int(speciesvalues[1]))


#calculate F1 score 2*((precision*recall)/(precision+recall))
phylaF1 = 2*((phylarecall*phylaprecision)/(phylaprecision+phylarecall))
classF1 = 2*((classrecall*classprecision)/(classprecision+classrecall))
orderF1 = 2*((orderrecall*orderprecision)/(orderprecision+orderrecall))
familyF1 = 2*((familyrecall*familyprecision)/(familyprecision+familyrecall))
genusF1 = 2*((genusrecall*genusprecision)/(genusprecision+genusrecall))
speciesF1 = 2*((speciesrecall*speciesprecision)/(speciesprecision+speciesrecall))


outputname =  ((args.t).split("/")[-1]).split("_TAXID")[0]
outputrecallname = str(outputname) + str("_RECALL_TAX.csv")
outputprecisionname = str(outputname) + str("_PRECISION_TAX.csv")
outputF1name = str(outputname) + str("_F1SCORE_TAX.csv")
if os.path.exists(outputrecallname):
	decision = 'a'
else:
	decision = 'w'
if os.path.exists(outputprecisionname):
	decision2 = 'a'
else:
	decision2 = 'w'
if os.path.exists(outputF1name):
	decision3 = 'a'
else:
	decision3 = 'w'
recalloutput = str(phylarecall) + "," + str(classrecall) + "," + str(orderrecall) + "," + str(familyrecall) + "," + str(genusrecall) + "," + str(speciesrecall) + "\n"
precisionoutput = str(phylaprecision) + "," + str(classprecision) + "," + str(orderprecision) + "," + str(familyprecision) + "," + str(genusprecision) + "," + str(speciesprecision) + "\n"
titleoutput = "phyla,class,order,family,genus,species" + '\n'
F1output = str(phylaF1) + "," + str(classF1) + "," + str(orderF1) + "," + str(familyF1) + "," + str(genusF1) + "," + str(speciesF1) + "\n"
with open(outputrecallname, decision) as l:
	l.write(titleoutput)
	l.write(recalloutput)


with open(outputprecisionname, decision2) as p:
	p.write(titleoutput)
	p.write(precisionoutput)

with open(outputF1name, decision3) as m:
	m.write(titleoutput)
	m.write(F1output)






