


from ete3 import NCBITaxa
import sys
import os
import csv
import argparse
parser = argparse.ArgumentParser(description='average OD data and normalise output')
parser.add_argument('-t', metavar='FILENAME', required=True, type=str, help="the truth file ")
parser.add_argument('-r', metavar='FILENAME', required=True, type=str, help="the metacache RESULTS query file (with the list of reads)")
args = parser.parse_args()

ncbi = NCBITaxa()

readscorrect = {"phylum": 0, "class": 0, "order":0, "family":0, "genus":0, "species":0}
readsincorrect = {"phylum": 0, "class": 0, "order":0, "family":0, "genus":0, "species":0}
totalcount = 0 #the number of simulated reads summed from th truth file
readanalysiscount = 0 #counting the lines of the kraken file (as each line is a read)

truth_data = open(args.t, 'r') 
readquery_data = open(args.r, 'r')
truthdict = {}
ranklist = ['phylum', 'class', 'order', 'family', 'genus', 'species']


if ((args.t).split("/")[-1]).split("_TAXID")[0] == ((args.r).split("/")[-1]).split("_RESULTS")[0]:
	print("REPORT and truth file match...starting analysis")
else:
	sys.exit('\033[91m' + "Truth file does not match report file" + '\033[0m')




reader = csv.reader(truth_data)
linecount = 0
for line in reader:
	if linecount > 0:
		totalcount += int(line[7])
		if line[9] not in ["0"]:
			truthdict[line[1]] = {k:v for k,v in ncbi.get_rank(ncbi.get_lineage(str(line[9]))).items() if v in ranklist} 
	linecount = linecount + 1

print("number of simulated reads in truth file " + str(totalcount))





krakenreader = csv.reader(readquery_data, delimiter='\t')
linecount2 = 0
for line in krakenreader:
	if linecount2 > 10:
		if line[0].startswith("G"):
			readanalysiscount += 1
			accession = line[0].split("_")[0]
			if line[2] != "--":
				querytax = (line[2].split(",")[0].split('(')[1].replace(")", ""))
				if txid not in ["0"]:
					taxid_lineage = {key:val for key,val in ncbi.get_rank(ncbi.get_lineage(str(txid))).items() if val in ranklist}

					comparativelineage = truthdict[accession]
					for tax, rank in taxid_lineage.items():
						for truthtax, truthrank in comparativelineage.items():
							if rank == truthrank:
								if  tax == truthtax:
									readscorrect[rank] += 1
								else:
									readsincorrect[rank] += 1
	linecount2 = linecount2 + 1

if totalcount != readanalysiscount:
	print("The truth file and the output file do not have the same number of reads (truth,kraken): " + str(totalcount) + "," + str(readanalysiscount))

speciesrecall = readscorrect["species"]/totalcount
genusrecall = readscorrect["genus"]/totalcount
familyrecall = readscorrect["family"]/totalcount
classrecall = readscorrect["class"]/totalcount
orderrecall = readscorrect["order"]/totalcount
phylarecall =readscorrect["phylum"]/totalcount

speciesprecision = readscorrect["species"]/(readscorrect["species"] + readsincorrect["species"])
genusprecision = readscorrect["genus"]/(readscorrect["genus"] + readsincorrect["genus"])
familyprecision = readscorrect["family"]/(readscorrect["family"] + readsincorrect["family"])
classprecision = readscorrect["class"]/(readscorrect["class"] + readsincorrect["class"])
orderprecision = readscorrect["order"]/(readscorrect["order"] + readsincorrect["order"])
phylaprecision = readscorrect["phylum"]/(readscorrect["phylum"] + readsincorrect["phylum"])

phylaF1 = 2*((phylarecall*phylaprecision)/(phylaprecision+phylarecall))
classF1 = 2*((classrecall*classprecision)/(classprecision+classrecall))
orderF1 = 2*((orderrecall*orderprecision)/(orderprecision+orderrecall))
familyF1 = 2*((familyrecall*familyprecision)/(familyprecision+familyrecall))
genusF1 = 2*((genusrecall*genusprecision)/(genusprecision+genusrecall))
speciesF1 = 2*((speciesrecall*speciesprecision)/(speciesprecision+speciesrecall))

#write to files
outputname = ((args.t).split("/")[-1]).split("_TAXID")[0]
recallname = str(outputname) + str("_RECALL_READLEVEL.csv")
precisionname = str(outputname) + str("_PRECISION_READLEVEL.csv")
F1name = str(outputname) + str("_F1SCORE_READLEVEL.csv")

recalloutput = str(phylarecall) + "," + str(classrecall) + "," + str(orderrecall) + "," + str(familyrecall) + "," + str(genusrecall) + "," + str(speciesrecall) + "\n"
precisionoutput = str(phylaprecision) + "," + str(classprecision) + "," + str(orderprecision) + "," + str(familyprecision) + "," + str(genusprecision) + "," + str(speciesprecision) + "\n"
titleoutput = "phyla,class,order,family,genus,species" + '\n'
F1output = str(phylaF1) + "," + str(classF1) + "," + str(orderF1) + "," + str(familyF1) + "," + str(genusF1) + "," + str(speciesF1) + "\n"


with open(recallname, 'w') as l:
	l.write(titleoutput)
	l.write(recalloutput)


with open(precisionname, 'w') as p:
	p.write(titleoutput)
	p.write(precisionoutput)

with open(F1name, 'w') as m:
	m.write(titleoutput)
	m.write(F1output)












	



