#!/usr/bin/env Rscript
args = commandArgs(trailingOnly=TRUE)

if (length(args)==0)  {
  stop("Arguments required, input file name and output file path", call.=FALSE)
}

file = read.csv(args[1], header=TRUE)

library("dplyr")
library("matrixStats")
library("ggplot2")
library("tidyverse")
library("RColorBrewer")


#get rid of the index column caled "X" and make new dataframe called df_bxp
df_bxp <- data.frame(subset(file, select=-c(X)))
df_bxp[ df_bxp == "na" ] <- NA
pdf(args[2])
title<-args[3]
type <- args[4]
print(title)

df_bxp$value <- as.numeric(as.character(df_bxp$value))


boxplot <- ggplot(df_bxp, aes(x=value, y=variable, color=tool))+ 
  geom_boxplot(outlier.shape=NA) + geom_point(size=0.2,pch=21,  position=position_jitterdodge(jitter.width=0.01)) + xlim(0, 100)


finallplot <- boxplot + scale_y_discrete(limits = c("Phylum", "Class", "Order", "Family", "Genus", "Species")) +
  ggtitle(title) + theme_classic() + theme(plot.title=element_text(hjust=0.5)) + scale_color_manual(values=c("#66c2a5", "#fc8d62", "#8da0cb", "#e78ac3", "#a6d854", "#ffd92f", "grey90")) +
  ylab("Rank") + xlab(type) + scale_fill_manual(values=c("#66c2a5", "#fc8d62", "#8da0cb", "#e78ac3", "#a6d854", "#ffd92f", "grey90"))


finallplot
dev.off() 



